/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SPoint } from "./SPoint";

/**
 * 线数据类型定义
 *
 * @author 庞利祥(sybotan@126.com)
 */
export class SLine {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** 端点1坐标 */
    p1: SPoint;
    /** 端点2坐标 */
    p2: SPoint;

    /** 属性X1 */
    get x1(): number {
        return this.p1.x;
    } // Get x1
    set x1(value: number) {
        this.p1.x = value;
    } // Set x1

    /** 属性Y1 */
    get y1(): number {
        return this.p1.y;
    } // Get y1
    set y1(value: number) {
        this.p1.y = value;
    } // Set y1

    /** 属性X2 */
    get x2(): number {
        return this.p2.x;
    } // Get x2
    set x2(v: number) {
        this.p2.x = v;
    } // Set x2

    /** 属性Y2 */
    get y2(): number {
        return this.p2.y;
    } // Get y2
    set y2(value: number) {
        this.p2.y = value;
    } // Set y2

    /** 属性dx */
    get dx(): number {
        return this.p2.x - this.p1.x;
    } // Get dx
    /** 属性dy */
    get dy(): number {
        return this.p2.y - this.p1.y;
    } // Get dy()

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 构造函数
    /**
     * 构造函数
     */
    constructor();

    /**
     * 构造函数
     *
     * @param   line    线段
     */
    constructor(line: SLine);

    /**
     * 构造函数
     *
     * @param   p1      启点坐标
     * @param   p2      终点坐标
     */
    constructor(p1: SPoint, p2: SPoint);

    /**
     * 构造函数
     *
     * @param   x1      启点X坐标
     * @param   y1      启点Y坐标
     * @param   x2      终点X坐标
     * @param   y2      终点Y坐标
     */
    constructor(x1: number, y1: number, x2: number, y2: number);

    /**
     * 构造函数（重载实现）
     *
     * @param   x1      启点X坐标 | 启点坐标 | 线段
     * @param   y1      启点Y坐标 | 终点坐标
     * @param   x2      终点X坐标
     * @param   y2      终点Y坐标
     */
    constructor(
        x1?: number | SPoint | SLine,
        y1?: number | SPoint,
        x2?: number,
        y2?: number
    ) {
        if (x1 == undefined) {
            this.p1 = new SPoint(0, 0);
            this.p2 = new SPoint(0, 0);
        } else if (x1 instanceof SLine) {
            this.p1 = new SPoint(x1.p1.x, x1.p1.y);
            this.p2 = new SPoint(x1.p2.x, x1.p2.y);
        } else if (x1 instanceof SPoint && y1 instanceof SPoint) {
            this.p1 = new SPoint(x1.x, x1.y);
            this.p2 = new SPoint(y1.x, y1.y);
        } else {
            this.p1 = new SPoint(x1 as number, y1 as number);
            this.p2 = new SPoint(x2 as number, y2 as number);
        }
    } // constructor

    /**
     * 判断线段是否为空
     *
     * @return  如果线段长度为零，返回true，否则返回false。
     */
    isNull(): boolean {
        return (
            Math.abs(this.p1.x - this.p2.x) + Math.abs(this.p1.y - this.p2.y) <
            0.000001
        );
    } // Function isNull()

    /**
     * 返回线段的中点
     *
     * @return  线段的中点
     */
    center(): SPoint {
        return new SPoint(
            (this.p1.x + this.p2.x) / 2,
            (this.p1.y + this.p2.y) / 2
        );
    } // Function center()

    /**
     * 平移线段
     *
     * @param   dx      X轴方向平衡量
     * @param   dy      Y轴方向平衡量
     */
    translate(dx: number, dy: number): void {
        this.p1.x += dx;
        this.p2.x += dx;
        this.p1.y += dy;
        this.p2.y += dy;
    } // Function translate()

    /**
     * 生成平移线段
     *
     * @param   dx      X轴方向平衡量
     * @param   dy      Y轴方向平衡量
     * @return  平移后的线段
     */
    translated(dx: number, dy: number): SLine {
        return new SLine(
            this.p1.x + dx,
            this.p1.y + dy,
            this.p2.x + dx,
            this.p2.y + dy
        );
    } // Function translate()
} // Class SLine
